@@ -65,6 +65,7 @@ TWEET_FIELDS = [
 ]
 
 MEDIA_FIELDS = [
+    "alt_text",
     "duration_ms",
     "height",
     "media_key",
@@ -91,22 +92,14 @@ PLACE_FIELDS = [
     "place_type",
 ]
 
-EVERYTHING = {
-    "expansions": ",".join(EXPANSIONS),
-    "user.fields": ",".join(USER_FIELDS),
-    "tweet.fields": ",".join(TWEET_FIELDS),
-    "media.fields": ",".join(MEDIA_FIELDS),
-    "poll.fields": ",".join(POLL_FIELDS),
-    "place.fields": ",".join(PLACE_FIELDS),
-}
-
-# For endpoints focused on user objects such as looking up users and followers.
-# Not all of the expansions are available for these endpoints.
-USER_EVERYTHING = {
-    "expansions": "pinned_tweet_id",
-    "tweet.fields": ",".join(TWEET_FIELDS),
-    "user.fields": ",".join(USER_FIELDS),
-}
+LIST_FIELDS = [
+    "owner_id",
+    "created_at",
+    "member_count",
+    "follower_count",
+    "private",
+    "description",
+]
 
 
 def extract_includes(response, expansion, _id="id"):
@@ -249,6 +242,15 @@ def ensure_flattened(data):
         log.warning(f"Unable to expand dictionary without includes: {data}")
         return flatten(data)
 
+    # If it's just an object with errors return an empty list
+    elif (
+        isinstance(data, dict)
+        and "data" not in data
+        and "includes" not in data
+        and "errors" in data
+    ):
+        return []
+
     # If it's a single response and both "includes" and "data" are missing, it is already flattened
     elif isinstance(data, dict) and "data" not in data and "includes" not in data:
         return [data]
